TillerTrackerAddon = LibStub("AceAddon-3.0"):NewAddon("TillerTracker", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TillerTracker")
local LibQTip = LibStub("LibQTip-1.0")

local _
local FOLDER_NAME, private = ...

local LDB = LibStub("LibDataBroker-1.1"):NewDataObject("TillerTracker",
{
	type 	= "data source",
	icon	= "Interface\\Icons\\Inv_Misc_Shovel_01",
	label	= L["Tiller Tracker"],
	text	= L["Tiller Tracker"],
	OnClick = function(clickedframe, button)
		if button == "LeftButton" then
			if IsAltKeyDown() then
				if (_G.Skillet) then
					
					-- Have Skillet show itself
					_G.Skillet:SkilletShow()
					
					-- Wait for Skillet to come up
					TillerTrackerAddon:ScheduleTimer("CheckSkillet", 0.25)
										
					return
				end
			end

			if (private.pinned) then
				private.tooltip:SetAutoHideDelay(0.25, self)

				private.pinned = false
			else	
				private.tooltip:SetAutoHideDelay()

				private.pinned = true
			end
		elseif button == "RightButton" then
			private.need_index = private.need_index + 1	
					
			TillerTrackerAddon:UpdateText()
		end
	end,
})

function TillerTrackerAddon:CheckSkillet()

	-- Check to see if Skillet is up yet
	if (_G.Skillet.tradeSkillFrame == nil or not _G.Skillet.tradeSkillFrame:IsVisible()) then

		-- Not yet - check again later
		TillerTrackerAddon:ScheduleTimer("CheckSkillet", 0.25)

		return	
	end

	-- Clear the current queue
	_G.Skillet:ClearQueue()

	-- Add all the recipes to the queue
	for _, quest_info in pairs(private.quest_table) do
		if (quest_info["CAN_CRAFT_COUNT"] > 0) then
			_G.Skillet:QueueAppendCommand(Skillet:QueueCommandIterate(quest_info["RECIPE_ID"], quest_info["CAN_CRAFT_COUNT"]))
		end
	end
end

function TillerTrackerAddon:OnInitialize()
	private.pinned = false
	private.need_index = 1
	private.loaded = false
	private.item_wait_table = {}
	private.item_wait_count = 0

	if not TillerTrackerDB then
		TillerTrackerDB = {}
		
		TillerTrackerDB["SORT_FIELD"] = "NAME"
		TillerTrackerDB["SORT_DIR"] = "ASC"
	end
end

function TillerTrackerAddon:OnEnable()
	-- Show loading text
	TillerTrackerAddon:Print(L["Loading..."])
	LDB.text = L["Loading..."]
	
	-- Register to get item info events for now
	self:RegisterEvent("GET_ITEM_INFO_RECEIVED")

	-- Attempt to load the cache
	TillerTrackerAddon:LoadCache()

	-- If everything is cached we're good to go
	if (private.item_wait_count == 0) then
		TillerTrackerAddon:OnLoaded(self)
	end
end

function TillerTrackerAddon:LoadCache()
	-- Loop over each quest in the data	
	for _, quest_info in pairs(private.QUESTS) do 

		-- Get the item ID of the food
		local food_id = quest_info["FOOD_ID"]				

		-- Try to get the info for the food
		local food_name = GetItemInfo(food_id)
		
		-- If the information wasn't available add it to the list of waiting items
		if (food_name == nil) then
			private.item_wait_table[food_id] = true
			private.item_wait_count = private.item_wait_count + 1
		end
		
		-- Loop over the mats required to cook the food for the quest
		for _, mat_data in pairs(quest_info["MATS"]) do 
		
			-- Get the item ID of the mat
			local mat_id = mat_data["ITEM_ID"]				

			-- Try to get the info for the mat
			local mat_name = GetItemInfo(mat_id)

			-- If the information wasn't available add it to the list of waiting items
			if (mat_name == nil) then
				if (private.item_wait_table[mat_id] == nil) then
					private.item_wait_table[mat_id] = true
					private.item_wait_count = private.item_wait_count + 1
				end
			end
		end
	end
end

function TillerTrackerAddon:GET_ITEM_INFO_RECEIVED()
	for id, _ in pairs(private.item_wait_table) do
		if GetItemInfo(id) then
			private.item_wait_table[id] = nil
			
			private.item_wait_count = private.item_wait_count - 1
		end
	end
	
	-- If everything is cached we're good to go
	if (private.item_wait_count == 0) then
		TillerTrackerAddon:OnLoaded(self)
	end
end

function TillerTrackerAddon:OnLoaded(self)
	-- Show loaded text
	TillerTrackerAddon:Print(L["Loaded"])
	LDB.text = L["Loaded"]
		
	-- Done with item info events
	self:UnregisterEvent("GET_ITEM_INFO_RECEIVED")

	self:RegisterEvent("QUEST_LOG_UPDATE")
	self:RegisterEvent("BAG_UPDATE")
	
	TillerTrackerAddon:UpdateData()
	TillerTrackerAddon:UpdateText()
	
	private.loaded = true	
end

local function Entry_OnMouseUp(frame, info, button)
	if button == "LeftButton" then
		if not _G.TomTom then
			return
		end	
	
		local location = info["LOCATION"]
	
		_G.TomTom:AddZWaypoint(6, 10, location[1], location[2], info["NAME"])
	end
end

local function SetSort(cell, sort)
	if TillerTrackerDB["SORT_FIELD"] == sort then
		if (TillerTrackerDB["SORT_DIR"] == "ASC") then
			TillerTrackerDB["SORT_DIR"] = "DESC"
		else
			TillerTrackerDB["SORT_DIR"] = "ASC"
		end
	else
		TillerTrackerDB["SORT_FIELD"] = sort
		TillerTrackerDB["SORT_DIR"] = "ASC"
	end

	TillerTrackerAddon:UpdateData()
	TillerTrackerAddon:UpdateTooltip()
end

function TillerTrackerAddon:UpdateData()
	-- Get information on all quests completed
	local questsCompleted = GetQuestsCompleted()

	-- Create a table to store our current inventory of required mats	
	local inv_table = {}
	
	-- Create a table to store all needs
	private.need_table = {}
	private.need_count = 0
	
	-- Loop over each quest in the data	
	for _, quest_info in pairs(private.QUESTS) do 
		
		-- Loop over the mats required to cook the food for the quest
		for _, mat_data in pairs(quest_info["MATS"]) do 
		
			-- Get the item ID of the mat
			local mat_id = mat_data["ITEM_ID"]				
			
			-- Figure out how many we have
			local have_count = GetItemCount(mat_id, true, false) 

			-- Add the count to the inventory table (in case of multiples this will just replace what is there)			
			inv_table[mat_id] = have_count
		end		
	end

	-- Create a table to hold the information to display for each quest
	private.quest_table = {}
	private.quest_count = 0

	-- Loop over each quest in the data
	for quest_id, quest_info in pairs(private.QUESTS) do 
	
		-- Get the reputation information
		local _, friendRep = GetFriendshipReputationByID(quest_info["REP_ID"])
		
		-- Only show quests where not at max rep
		if not (friendRep > 42000) then
		
			-- Get the ID of the food required for this quest
			local food_id = quest_info["FOOD_ID"]

			-- Get the name of the food for display
			local food_name = GetItemInfo(food_id)

			-- Get how many of the food we have in our bags
			local food_count = GetItemCount(food_id, true, false)

			-- Color the food count based on whether there is enough in the bags or not
			local food_count_display = (food_count >= 5) and "|cFF00FF00" .. food_count .. "|r" or "|cFFFFFF00" .. food_count .. "|r"

			-- Initialize need and status strings
			local need = ""
			local need_count = 0
			local quest_status = ""
			local can_craft = 999999

			if (questsCompleted[quest_id]) then
				-- Quest has already been completed today - nothing else to do
				quest_status = L["|cFF00FF00Complete|r"]
			else
				-- Quest still needs to be done
				quest_status = L["|cFFFF0000Not Complete|r"]

				-- Figure out how many of the food we need to get to 5		
				need_count = (food_count < 5) and 5 - food_count or 0

				-- If some food is still needed then display what mats are required
				if (need_count > 0) then

					-- Loop over all mats for this food
					for _, mat_data in pairs(quest_info["MATS"]) do 

						-- Get the mat ID 
						local mat_id = mat_data["ITEM_ID"]
						
						-- Figure out how many we can make with this mat
						local mat_can_craft = math.floor(inv_table[mat_id] / mat_data["COUNT"])
						
						-- The smaller value (current or the mat) is what we can create
						can_craft = math.min(can_craft, mat_can_craft)

						-- Figure out how many of this mat we need
						local mat_count = (mat_data["COUNT"] * need_count) - inv_table[mat_id]

						-- Deduct the mats used to make the item from the inventory table
						inv_table[mat_id] = inv_table[mat_id] - (mat_data["COUNT"] * need_count)

						-- If the stock of the item went below 0 then reset it to 0
						if (inv_table[mat_id] < 0) then 
							inv_table[mat_id] = 0 
						end				

						-- See if we need mats
						if (mat_count > 0) then

							-- Get the name of the mat					
							local mat_name = GetItemInfo(mat_id)

							-- Set the mat need string to the count needed and the name
							local mat_need = mat_count .. " " .. mat_name

							if private.need_table[mat_id] == nil then
								private.need_table[mat_id] = mat_count
								private.need_count = private.need_count + 1
							else
								private.need_table[mat_id] = private.need_table[mat_id] + mat_count
							end

							-- Add this mat to the list for this quest
							if (need == "") then
								need = mat_need
							else
								need = need .. ", " .. mat_need
							end
						end
					end
				end
			end

			-- Insert everything we figured out into the quest table
			table.insert(private.quest_table,
								{
								NAME = quest_info["NAME"],
								NAME_DISPLAY = "|cffeda55f" .. quest_info["NAME"] .. "|r",
								FOOD = food_name,
								FOOD_DISPLAY = food_name,
								AMOUNT = food_count,
								AMOUNT_DISPLAY = food_count_display,
								STATUS = quest_status,
								LOCATION = quest_info["LOCATION"],
								NEED = need,
								NEED_COUNT = need_count,
								RECIPE_ID = quest_info["RECIPE_ID"],
								CAN_CRAFT_COUNT = can_craft
								})
			
			private.quest_count = private.quest_count + 1
		end
	end

	-- Sort the quest table using the sort field and direction from the settings	
	table.sort(private.quest_table, private.SORT_FUNCTIONS[TillerTrackerDB["SORT_FIELD"] .. "_" .. TillerTrackerDB["SORT_DIR"]])
end

function TillerTrackerAddon:UpdateTooltip()
	if LibQTip:IsAcquired("TillerTracker") then
		private.tooltip:Clear()
	else
		private.tooltip = LibQTip:Acquire("TillerTracker", 5)
		
		private.tooltip:SetBackdropColor(0, 0, 0, 1)

		private.tooltip:SmartAnchorTo(private.LDB_ANCHOR)
		private.tooltip:SetAutoHideDelay(0.25, private.LDB_ANCHOR)
	end
	
	-- If there are no quests show just the status line
	if (private.quest_count == 0) then
		line = private.tooltip:AddHeader()
		private.tooltip:SetCell(line, 1, L["Best Friends with everyone"], "LEFT", private.tooltip:GetColumnCount())			
	
		return
	end
	
	-- Add the header line and set the cell text and scripts
	local line = private.tooltip:AddHeader()
	
	private.tooltip:SetCell(line, 1, L["Quest"])
	private.tooltip:SetCellScript(line, 1, "OnMouseUp", SetSort, "NAME")
	
	private.tooltip:SetCell(line, 2, L["Food"])
	private.tooltip:SetCellScript(line, 2, "OnMouseUp", SetSort, "FOOD")
	
	private.tooltip:SetCell(line, 3, L["Have"])
	private.tooltip:SetCellScript(line, 3, "OnMouseUp", SetSort, "AMOUNT")
	
	private.tooltip:SetCell(line, 4, L["Status"])
	private.tooltip:SetCellScript(line, 4, "OnMouseUp", SetSort, "STATUS")
	
	private.tooltip:SetCell(line, 5, L["Need"])
	private.tooltip:SetCellScript(line, 5, "OnMouseUp", SetSort, "NEED")

	private.tooltip:AddSeparator()
	
	-- Loop over all quests in the sorted table
	for _, quest in pairs(private.quest_table) do
	
		-- Add the line with info about the current quest
		line = private.tooltip:AddLine(quest["NAME_DISPLAY"], quest["FOOD_DISPLAY"], quest["AMOUNT_DISPLAY"], quest["STATUS"], quest["NEED"])

		-- If TomTom is installed then set the line script
		if _G.TomTom then	
			private.tooltip:SetLineScript(line, "OnMouseUp", Entry_OnMouseUp, { NAME = quest["NAME"], LOCATION = quest["LOCATION"] })
		end
	end				

	-- Add the normal usage hints
	private.tooltip:AddLine(" ")
	
	line = private.tooltip:AddLine()
	private.tooltip:SetCell(line, 1, L["|cFFFFFF00Click|r the main button to toggle whether the tooltip stays open"], "LEFT", private.tooltip:GetColumnCount())
	
	line = private.tooltip:AddLine()
	private.tooltip:SetCell(line, 1, L["|cFFFFFF00Right Click|r the main button to cycle through which ingredient to track"], "LEFT", private.tooltip:GetColumnCount())

	-- If TomTom is installed then add a hint to the end
	if _G.TomTom then		
		line = private.tooltip:AddLine()
		private.tooltip:SetCell(line, 1, L["|cFFFFFF00Click|r a quest in the tooltip to set a TomTom waypoint at the farmer's home"], "LEFT", private.tooltip:GetColumnCount())
	end
	
	-- Add the Skillet hint if installed
	if _G.Skillet then
		line = private.tooltip:AddLine()
		private.tooltip:SetCell(line, 1, L["|cFFFFFF00Alt-Click|r the main button to queue craftable foods in Skillet"], "LEFT", private.tooltip:GetColumnCount())		
	end
end

function TillerTrackerAddon:UpdateText()
	local need_index = 1
	
	if (private.need_index > private.need_count) then
		private.need_index = 1
	end
	
	if (private.need_count == 0) then
		LDB.text = L["Done"]
		return
	end

	-- Loop over the need table
	for need_id, need_count in pairs(private.need_table) do
		if need_index == private.need_index then
			LDB.text = L["Gather:"] .. " " .. need_count .. " " .. GetItemInfo(need_id)
			
			return
		end
		
		need_index = need_index + 1
	end
end

function LDB.OnEnter(self)
	if not private.loaded then
		return
	end

	private.LDB_ANCHOR = self
	TillerTrackerAddon:UpdateTooltip()
	
	private.tooltip:Show()
end

function TillerTrackerAddon:QUEST_LOG_UPDATE()
	TillerTrackerAddon:UpdateData()
	TillerTrackerAddon:UpdateText()
	
	if not private.LDB_ANCHOR then
		return
	end
	
	TillerTrackerAddon:UpdateTooltip()
end

function TillerTrackerAddon:BAG_UPDATE()
	TillerTrackerAddon:UpdateData()
	TillerTrackerAddon:UpdateText()

	if not private.LDB_ANCHOR then
		return
	end

	TillerTrackerAddon:UpdateTooltip()
end
